@@ -32,7 +32,7 @@ Box_Data_Details_Find::Box_Data_Details_Find()
   //Instead of nav buttons:
   m_HBox.pack_end(m_Button_Find, Gtk::PACK_SHRINK);
 
-  m_Button_Find.property_can_default() = true; //TODO: Make this a real method in gtkmm?
+  g_object_set(m_Button_Find.gobj(), "can-default", TRUE, (gpointer)NULL); //TODO: Make this a real method in gtkmm?
 
   show_all_children();
 }
@@ -52,7 +52,7 @@ bool Box_Data_Details_Find::fill_from_database()
 {
   Bakery::BusyCursor busy_cursor(get_app_window());
 
-  const bool result = Box_DB_Table::fill_from_database();
+  const bool result = Base_DB_Table_Data::fill_from_database();
   if(!result)
     return result;
 
