@@ -20,6 +20,7 @@
 
 //We need to include this before anything else, to avoid redefinitions:
 #include <glom/libglom/python_embed/py_glom_record.h>
+#include <glom/libglom/python_embed/pygdavalue_conversions.h>
 
 #define NO_IMPORT_PYGTK //To avoid a multiple definition in pygtk.
 #include <pygtk/pygtk.h> //For the PyGObject and PyGBoxed struct definitions.
@@ -32,6 +33,9 @@
 #include <glom/libglom/data_structure/glomconversions.h>
 #include <list>
 #include <glib.h> //For g_warning().
+#include <glibmm/i18n.h>
+
+#include <gtkmm/messagedialog.h>
 
 namespace Glom
 {
@@ -59,12 +63,79 @@ std::list<Glib::ustring> ustring_tokenize(const Glib::ustring& msg, const Glib::
   return result;
 }
 
+// Use this for errors not (directly) caused by the user
 void HandlePythonError()
 {
   if(PyErr_Occurred())
     PyErr_Print();
 }
 
+// Show python coding errors of the user
+void ShowTrace()
+{
+  // Python equivilant:
+  // import traceback, sys
+  // return "".join(traceback.format_exception(sys.exc_type,
+  //    sys.exc_value, sys.exc_traceback))
+
+  PyObject *type, *value, *traceback;
+
+  PyErr_Fetch(&type, &value, &traceback);
+  
+  if (!traceback)
+  {
+    std::cerr << "traceback = 0" << std::endl;
+  }
+
+  PyObject *tracebackModule = PyImport_ImportModule((char*)"traceback");
+  gchar* chrRetval = 0;
+  if (tracebackModule != NULL)
+  {
+      PyObject* tbList = PyObject_CallMethod(
+          tracebackModule,
+          (char*)"format_exception",
+          (char*)"OOO",
+          type,
+          value == NULL ? Py_None : value,
+          traceback == NULL ? Py_None : traceback);
+      
+      if (!tbList)
+      {
+        std::cerr << "format_exception failed" << std::endl;
+        return;
+      }
+
+      PyObject* emptyString = PyString_FromString("");
+      PyObject* strRetval = PyObject_CallMethod(emptyString, (char*)"join",
+            (char*)"O", tbList);
+      if (strRetval)
+        chrRetval = g_strdup(PyString_AsString(strRetval));
+    
+      Py_DECREF(tbList);
+      Py_DECREF(emptyString);
+      Py_DECREF(strRetval);
+      Py_DECREF(tracebackModule);
+    }
+    else
+    {
+        std::cerr << "Unable to import traceback module." << std::endl;
+        
+    }
+
+    Py_DECREF(type);
+    Py_XDECREF(value);
+    Py_XDECREF(traceback);
+  
+  if (chrRetval)
+  {
+    Glib::ustring message = _("Python Error: \n");
+    message += chrRetval;
+    Gtk::MessageDialog dialog(message, false, Gtk::MESSAGE_ERROR);
+    dialog.run();
+  }
+  g_free(chrRetval);
+}
+
 void glom_execute_python_function_implementation(const Glib::ustring& func_impl, const type_map_fields& field_values, Document_Glom* pDocument, const Glib::ustring& table_name, const Glib::RefPtr<Gnome::Gda::Connection>& opened_connection)
 {
   glom_evaluate_python_function_implementation(Field::TYPE_TEXT, func_impl, field_values, pDocument, table_name, opened_connection);
@@ -101,19 +172,20 @@ Gnome::Gda::Value glom_evaluate_python_function_implementation(Field::glom_field
   func_def = "def " + func_name + "(record):\n  import glom\n  import gda\n" + func_def;
   //We did this in main(): Py_Initialize();
 
-  PyObject* pMain = PyImport_AddModule("__main__");
+  PyObject* pMain = PyImport_AddModule((char*)"__main__");
   PyObject* pDict = PyModule_GetDict(pMain);
 
 
   //Allow the function to import from our script library:
-  const std::vector<Glib::ustring> module_names = pDocument->get_library_module_names();
-  for(std::vector<Glib::ustring>::const_iterator iter = module_names.begin(); iter != module_names.end(); ++iter)
+  if (pDocument)
   {
-    const Glib::ustring name = *iter;
-    const Glib::ustring script = pDocument->get_library_module(name);
-    if(!name.empty() && !script.empty())
+    const std::vector<Glib::ustring> module_names = pDocument->get_library_module_names();
+    for(std::vector<Glib::ustring>::const_iterator iter = module_names.begin(); iter != module_names.end(); ++iter)
     {
-
+      const Glib::ustring name = *iter;
+      const Glib::ustring script = pDocument->get_library_module(name);
+      if(!name.empty() && !script.empty())
+      {
         PyObject* objectCompiled = Py_CompileString(script.c_str(), name.c_str() /* "filename", for debugging info */,  Py_file_input /* "start token" for multiple lines of code. */); //Returns a reference.
   
         if(!objectCompiled)
@@ -122,34 +194,47 @@ Gnome::Gda::Value glom_evaluate_python_function_implementation(Field::glom_field
         PyObject* pObject = PyImport_ExecCodeModule(const_cast<char*>(name.c_str()), objectCompiled); //Returns a reference. //This should make it importable.
 
         if(!pObject)
-          HandlePythonError();
+         HandlePythonError();
 
         Py_DECREF(pObject);
         Py_DECREF(objectCompiled);
         //TODO: When do these stop being importable? Should we unload them somehow later?
+      }
     }
   }
 
 
-  PyObject* module_glom = PyImport_ImportModule("glom");
+  PyObject* module_glom = PyImport_ImportModule((char*)"glom");
   if(!module_glom)
+  {
     g_warning("Could not import python glom module.");
+    return valueResult; // don't crash
+  }
 
   PyObject* module_glom_dict = PyModule_GetDict(module_glom);
   //This seems to be different to PyGlomRecord_GetPyType() - we can PyObject_Call() this one to instantiate it.
-  PyObject* pyTypeGlomRecord = PyDict_GetItemString(module_glom_dict, "Record"); //TODO: Unref this?
+  PyObject* pyTypeGlomRecord = PyDict_GetItemString(module_glom_dict, (char*)"Record"); //TODO: Unref this?
   if(!pyTypeGlomRecord || !PyType_Check(pyTypeGlomRecord))
+  {
     g_warning("Could not get glom.Record from glom_module.");
+    return valueResult; // don't crash
+  }
 
 
-  PyObject* module_gda = PyImport_ImportModule("gda");
+  PyObject* module_gda = PyImport_ImportModule((char*)"gda");
   if(!module_gda)
+  {
     g_warning("Could not import python gda module.");
+    return valueResult;
+  }
 
+  // Gda.Value does not exist anymore in pygda-3.0
+#if 0
   PyObject* module_gda_dict = PyModule_GetDict(module_gda);
   PyObject* pyTypeGdaValue = PyDict_GetItemString(module_gda_dict, "Value"); //TODO: Unref this?
   if(!pyTypeGdaValue || !PyType_Check(pyTypeGdaValue))
     g_warning("Could not get gda.Value from gda_module.");
+#endif
 
 
   //Create the function definition:
@@ -160,16 +245,21 @@ Gnome::Gda::Value glom_evaluate_python_function_implementation(Field::glom_field
     pyValue = 0;
   }
   else
-    HandlePythonError();
+  {
+    ShowTrace();
+  }
 
   //Call the function:
   {
     PyObject* pFunc = PyDict_GetItemString(pDict, func_name.c_str()); //The result is borrowed, so should not be dereferenced.
     if(!pFunc)
+    {
       HandlePythonError();
+    }
 
     if(!PyCallable_Check(pFunc))
     {
+      HandlePythonError();
       g_warning("pFunc is not callable.");
       return valueResult;
     }
@@ -206,35 +296,20 @@ Gnome::Gda::Value glom_evaluate_python_function_implementation(Field::glom_field
         //Deal with the various possible return types:
         bool object_is_gda_value = false;
 
-        //This is a hack - see below:
-        //if( strcmp(pyResult->ob_type->tp_name, "gda.Value") == 0 )
-        //  object_is_gda_value = true;
+        GValue value = { 0 };
+        const int test = pygda_value_from_pyobject(&value, pyResult);
 
-        //g_warning("debug: pyResult->ob_type->tp_name=%s", pyResult->ob_type->tp_name);
-        //g_warning("debug: pyTypeGdaValue->tp_name=%s", ((PyTypeObject*)pyTypeGdaValue)->tp_name);
+        if(test == 0) //-1 means error.
+	  object_is_gda_value = true;
 
-        const int test = PyType_IsSubtype(pyResult->ob_type, (PyTypeObject*)pyTypeGdaValue);
-        if(test == -1) //-1 means error.
-          HandlePythonError();
-
-        if(test == 1) // 0 means false, -1 means error.
-          object_is_gda_value = true;
-
-        if(object_is_gda_value)
+        if(object_is_gda_value && G_IS_VALUE(&value))
         {
-          //Cast it to the "derived" struct type:
-          //All boxed types are wrapped with PyGBoxed in pygtk, without their own special structs,
-          //and GValue is a boxed-type.
-          PyGBoxed* pygtkobject = (PyGBoxed*)pyResult;
-          if(pygtkobject)
-          {
-            GdaValue* cboxed = (GdaValue*)pygtkobject->boxed;
-            if(cboxed)
-              valueResult = Glib::wrap(cboxed, true /* take_copy */);
-            g_warning("PyGBoxed::boxed is null");
-          }
-          else
-            g_warning("PyGBoxed is null");
+	  valueResult = Gnome::Gda::Value(&value);
+          //Make sure that the value is of the expected Gda type:
+          //TODO_Performance:
+          valueResult = Glom::Conversions::convert_value(valueResult, result_type);
+          //std::cout << "DEBUG: glom_evaluate_python_function_implementation(): valueResult Gda type=" << g_type_name(valueResult.get_value_type()) << std::endl;
+	  g_value_unset(&value);
         }
         else
         {
@@ -244,12 +319,15 @@ Gnome::Gda::Value glom_evaluate_python_function_implementation(Field::glom_field
           //instead of returning 0.
           if(pyResult == Py_None) //Direct comparison is possible and recommended, because there is only one pyNone object.
           {
+ 
             //The result should be an appropriate empty value for this field type:
             valueResult = Conversions::get_empty_value(result_type);
+            //std::cout << "DEBUG: glom_evaluate_python_function_implementation(): empty value Gda type=" << g_type_name(valueResult.get_value_type()) << std::endl;
           }
           else
           {
             //TODO: Handle numeric/date/time types:
+            //(though I don't think this code is ever reached. murrayc)
 
             //Treat this as a string or something that can be converted to a string:
             PyObject* pyObjectResult = PyObject_Str(pyResult);
@@ -260,12 +338,14 @@ Gnome::Gda::Value glom_evaluate_python_function_implementation(Field::glom_field
               {
                 bool success = false;
                 valueResult = Conversions::parse_value(result_type, pchResult, success, true /* iso_format */);
+                std::cout << "DEBUG: glom_evaluate_python_function_implementation(): parsed value Gda type=" << g_type_name(valueResult.get_value_type()) << std::endl;
+ 
               }
               else
-                g_warning("pchResult is null");
+                HandlePythonError();
             }
             else
-              g_warning("PyString_Check returned false");
+              HandlePythonError();
           }
         }
 
